@@ -22,7 +22,10 @@
 
 // Qt
 #include <QWidget>
+#include <QGraphicsView>
+#include <QTimer>
 
+class DummyWidget;
 class QGraphicsScene;
 namespace Plasma
 {
@@ -31,8 +34,7 @@ class FrameSvg;
 
 class QHBoxLayout;
 class KimpanelInputPanelGraphics;
-class QGraphicsView;
-class KimpanelInputPanel : public QWidget
+class KimpanelInputPanel : public QGraphicsView
 {
     Q_OBJECT
 public:
@@ -56,6 +58,7 @@ public:
                         bool hasNext,
                         const QStringList& attrs = QStringList()
                        );
+    void updateSizeVisibility();
 
 Q_SIGNALS:
     void lookupTablePageUp();
@@ -64,21 +67,20 @@ Q_SIGNALS:
 protected:
     virtual void showEvent(QShowEvent* event);
     virtual void resizeEvent(QResizeEvent* event);
-    virtual void paintEvent(QPaintEvent* event);
+    virtual void drawBackground(QPainter* painter, const QRectF& rect);
 private Q_SLOTS:
     void loadTheme();
     void maskBackground(bool);
     void updateVisible(bool);
     void updateSize();
 private:
-    QHBoxLayout* m_layout;
-    QGraphicsScene* m_scene;
-    QGraphicsView* m_view;
     KimpanelInputPanelGraphics* m_widget;
     Plasma::FrameSvg* m_backgroundSvg;
     QPoint m_pointPos;
     bool m_moving;
     QRect m_spotRect;
+    bool m_composite;
+    bool m_useBlur;
 };
 
 #endif // KIMPANEL_INPUTPANEL_H
